@@ -6,7 +6,7 @@ import shlex
 import socket
 import subprocess
 from json import JSONDecodeError
-from subprocess import TimeoutExpired
+from pathlib import Path
 from typing import Union
 
 import plogger
@@ -133,9 +133,9 @@ class Plinux:
             err = result.stderr.decode().strip()
 
             return SSHResponse(out, err, result.returncode, cmd)
-        except subprocess.TimeoutExpired:
+        except subprocess.TimeoutExpired as err:
             self.logger.exception('Connection timeout')
-            raise TimeoutExpired
+            raise err
 
     def run_cmd_local_native(self, *popenargs,
                              stdin_input=None,
@@ -202,23 +202,28 @@ class Plinux:
                 return client.open_sftp()
             return client
         except ssh_exception.AuthenticationException as err:
-            self.logger.exception('Authentication error')
+            msg = f'{self.host:<14}| Invalid credentials: {self.username}@{self.password}:\n{err}.'
+            self.logger.exception(msg)
             raise err
         except ssh_exception.NoValidConnectionsError as err:
-            # Fixme add more sophisticated verification
-            self.logger.exception('There is no valid connection. Try to use "_local" or vice versa method.')
+            msg = f'{self.host:<14}| There is no valid connection. Try to use "_local" or vice versa method.'
+            self.logger.exception(msg)
             raise err
         except TimeoutError as err:
-            self.logger.exception('Timeout exceeded.')
+            self.logger.exception(f'{self.host:<14}| Timeout exceeded.')
+            raise err
+        except Exception as err:
+            self.logger.exception(f'{self.host:<14}| Something went wrong:\n{err}.')
             raise err
 
-    def run_cmd(self, cmd: str, sudo: bool = False, timeout: int = 30) -> SSHResponse:
+    def run_cmd(self, cmd: str, sudo: bool = False, timeout: int = 30, ignore_errors: bool = False) -> SSHResponse:
         """Base method to execute SSH command on remote server
 
         Args:
             cmd: SSH command
             sudo: Execute specified command as sudo user
             timeout: Execution timeout
+            ignore_errors: Ignore errors
 
         Returns:
             SSHResponse class
@@ -241,22 +246,21 @@ class Plinux:
             err_ = err.read().decode().strip()
             parsed = SSHResponse(out=out_, err=err_, exited=exited, command=cmd)
 
+            # Log ERROR and WARNING
+            msg_err = f'{self.host:<14} | {parsed.exited}: Something went wrong!\n\tSTDERR: {parsed.stderr}'
+            if parsed.exited and not ignore_errors:  # Exit code != 0
+                self.logger.error(msg_err)
+                raise RemoteCommandExecutionError(parsed.stderr)
+
+            # Log WARNING
+            if parsed.stderr:  # Exit code != 0 and stderr contains message
+                self.logger.warning(msg_err)  # Exit code != 0 and no stderr
+
             # Log response (INFO)
             out_to_log = json.dumps(parsed.stdout, indent=4) if isinstance(parsed.stdout, dict) else parsed.stdout
             msg_to_log = f'{parsed.exited}:\n\t{out_to_log}' if out_to_log else f'{parsed.exited}:'
             self.logger.info(f'{self.host:<14} | {msg_to_log}')
 
-            # Log ERROR and WARNING
-            if parsed.exited:  # Exit code != 0
-                msg_err = f'{self.host:<14} | Something went wrong!'
-
-                if parsed.stderr:  # Exit code != 0 and stderr contains message
-                    msg_err += f'\n\t{parsed.stderr}'
-                    self.logger.error(msg_err)
-                    raise RemoteCommandExecutionError(parsed.stderr)
-
-                self.logger.warning(msg_to_log)
-                return parsed  # Exit code != 0 and no stderr
             return parsed
         finally:
             client.close()
@@ -724,11 +728,15 @@ class Plinux:
         :return:
         """
 
+        self.logger.info(f'-> Verify entity ({path}) existence')
+
         # Linux
         if '/' in path:
             cmd = f'test -e {path}'
-            response = self.run_cmd(cmd, sudo=sudo)
-            return response.ok
+            response = self.run_cmd(cmd, sudo=sudo, ignore_errors=True)
+            result = response.ok
+            self.logger.info(f'<- {result}')
+            return result
         # Windows
         elif '\\' in path:
             return os.path.exists(path)
@@ -943,6 +951,37 @@ class Plinux:
 
         return result.stdout
 
+    def compare_files(self, file1: str | Path, file2: str | Path, params: str = '', sudo: bool = False) -> bool:
+        """Compare two files byte by byte.
+
+        Command:
+            - cmp -s {params} {file1} {file2}
+
+        Args:
+            file1: /home/objectfirst/file_bin_1
+            file2: /home/objectfirst/file_text_2
+            params:
+            sudo:
+
+        Returns:
+            Bool
+        """
+
+        self.logger.info(f'-> Compare two files byte by byte\n\t1. {file1}\n\t2. {file2}')
+
+        cmd = f'cmp -s {params} {file1} {file2}'
+        result = self.run_cmd(cmd, sudo=sudo, ignore_errors=True)
+
+        if not result.exited:
+            self.logger.info('<- Files are equal')
+            return True
+        elif result.exited == 1:
+            self.logger.info('<- Files are different')
+            return False
+        else:
+            self.logger.error('<- File(s) not found')
+            raise FileNotFoundError(f'"{file1}" or "{file2}" not found')
+
     def remove(self, path: str, sudo: bool = True) -> bool:
         """Remove file(s) and directories
 
@@ -1189,8 +1228,9 @@ class Plinux:
         try:
             return int(result)
         except TypeError as err:
-            self.logger.exception(f'Path ({path}) not found or something went wrong.')
-            raise err
+            msg = f'Path ({path}) not found or something went wrong.'
+            self.logger.exception(msg)
+            raise RemoteCommandExecutionError(msg) from err
 
     def get_dir_size(self, path: str, sudo: bool = False) -> int:
         """Get directory size
@@ -1211,10 +1251,10 @@ class Plinux:
 
         try:  # Use try-block to process invalid paths. Returns string if path is invalid
             return int(result)
-        except ValueError:
+        except ValueError as err:
             msg = f'Path ({path}) not found or something went wrong.'
             self.logger.exception(msg)
-            raise RemoteCommandExecutionError(msg)
+            raise RemoteCommandExecutionError(msg) from err
 
     def get_port_listeners_process_id(self, port: int) -> Union[int, list[int]]:
         r"""Get processes ids of port listeners
@@ -1230,8 +1270,9 @@ class Plinux:
         try:
             return int(result)
         except TypeError as err:
-            self.logger.exception(f'Path ({port}) not found or something went wrong.')
-            raise err
+            msg = f'Port ({port}) not found or something went wrong.'
+            self.logger.exception(msg)
+            raise RemoteCommandExecutionError(msg) from err
         except ValueError:
             return list(map(int, result.splitlines()))
 
@@ -1457,11 +1498,11 @@ def to_known_type(value: str):
 
     if value_lower in ('yes', 'true'):
         return True
-    elif value_lower in ('no', 'false'):
+    if value_lower in ('no', 'false'):
         return False
-    elif value_lower in ('none', 'empty', ''):
+    if value_lower in ('none', 'empty', ''):
         return None
-    elif value_lower.startswith('00'):
+    if value_lower.startswith('00'):
         return value
 
     try:
